SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO


ALTER  procedure usp_delzerosift
  @runhours tinyint = 1,         --Hours to run the script, defaults to 1
  @backup nvarchar(500) = N''    -- backup device

/* 
 The backup parameter will define the backup device or the truncate command
 to truncate the transaction log during the runtime send @backup = 'WITH NO_LOG' but remember 
  this will break log backup continouity and that backup log [db] with NO_LOG will not work in sql server 2008
  
 I Rather recomend to send the name of a backup device or a file to make the backup in the following form
 @backup = N'to DISK = '''c:\backup.bak''' 
 or @backup = N'to BACKUPDevice'
 The Default behavior is not to make log backups during run
*/

 as

declare @sift_tab_id int,
		@stoptime datetime,
        @fld_name varchar(20), 
        @Delete_Statement nvarchar(1000),
        @tname nvarchar(128),
        @Count_SIFT_Records_QUERY nvarchar(1000),
        @Count_SIFT_Zero_Records_Query nvarchar(1000),
        @SIFT_Where_Clause nvarchar(1000),
        @fldno_total int, 
        @fldno int,
        @fld_size int,
        @rec_curr int,
        @sift_total_rec int,
        @ParmDefinition nvarchar(500),
        @sift_zero_rec int,
        @backupcmd nvarchar(1000),
        @deleted_records int,
        @total_deleted_records int

set @rec_curr = 0
set @sift_total_rec = 0
set @sift_zero_rec = 0
set @deleted_records = 0
set @total_deleted_records = 0
set nocount on						 
set statistics io off
SET DEADLOCK_PRIORITY LOW -- If we are in a deadlock chain always yield and become the victim

set @stoptime = dateadd (hh, @runhours, getdate())

-- temporary table for sorting by table-name
create table #sift_tab ([tab_id] int, [tab_name] varchar(250))
insert into #sift_tab select [id], object_name([id]) from dbo.sysobjects where 
(objectproperty([id], N'IsUserTable') = 1) and (object_name([id]) like ('%$' + '[0-9]' + '%')) and right(object_name([id]), 1) in ('0','1','2','3','4','5','6','7','8','9')

-- cursor SIFT tables
-- This is modified from Jrg A Stryk's procedures, any errors are mine
--chech out the Stryk.info webpage for information about Stryks services
declare sift_tab_cur cursor fast_forward for select [tab_id] from #sift_tab order by [tab_name]
open sift_tab_cur
fetch next from sift_tab_cur into @sift_tab_id
while @@fetch_status = 0 begin
if @stoptime < getdate() goto End_Step
  -- check for valid SIFT record
  if (not exists(select [name] from dbo.syscolumns where (id = @sift_tab_id) and ([name] like 's' + '[0-9]' + '%')))
    or
     (not exists(select [name] from dbo.syscolumns where (id = @sift_tab_id) and ([name] like 'f' + '[0-9]' + '%')))
    or
     (exists(select [name] from dbo.syscolumns where (id = @sift_tab_id) and ([name] like 's' + '[0-9]' + '%') and (type_name([xtype]) <> 'decimal')))
    or
     (exists(select [name] from dbo.syscolumns where (id = @sift_tab_id) and ([name] = 'timestamp')))
    goto End_Step
/*   
 We have a SIFT table
 Now lets get the total number of SIFT records
*/
	 SET @tname = OBJECT_NAME(@sift_tab_id)
	 SET @Count_SIFT_Records_QUERY = N'SELECT @sift_total_rec = (select COUNT (*) from ['+@tname+'])'
     SET @ParmDefinition = N'@sift_total_rec int OUTPUT'
	 execute sp_executesql @Count_SIFT_Records_QUERY
		,@ParmDefinition
		,@sift_total_rec output

-- if we have 0 SIFT records lets move on 
   if @sift_total_rec = 0 goto End_Step

/*
 Build the WHERE clause used to count and then delete the SIFT records
 Some day this will fail because the old system tables are going out of style
*/

  set @SIFT_Where_Clause = N' WHERE ('
  select @fldno_total = count([id]) from dbo.syscolumns where id = @sift_tab_id and [name] like 's%'
  set @fldno = 1
  declare fld_cur cursor fast_forward for select [name] from dbo.syscolumns where id = @sift_tab_id and [name] like 's%'
  open fld_cur
  fetch next from fld_cur into @fld_name
  while @@fetch_status = 0 begin
        set @SIFT_Where_Clause = @SIFT_Where_Clause + '(' + @fld_name + ' = 0)'
    if (@fldno >= 1) and (@fldno < @fldno_total) begin
      set @SIFT_Where_Clause = @SIFT_Where_Clause + ' AND '
    end
    set @fldno = @fldno + 1    
    fetch next from fld_cur into @fld_name
  end
  close fld_cur
  deallocate fld_cur
  set @SIFT_Where_Clause = @SIFT_Where_Clause + ')'
 
/*
 Now we count the number of lines where all SIFT Sums are zero, Then we decide if we should delete the records or just leave them be. 
 An arbituary decision was made not to delete from a table if less than 10% of the SIFT sums are Zero.
*/
SET @Count_SIFT_Zero_Records_Query = N'select @sift_zero_rec = (SELECT COUNT (*) FROM ['+@tname +']' +@SIFT_Where_Clause +')'
SET @ParmDefinition = N'@sift_zero_rec int OUTPUT';
 execute sp_executesql @Count_SIFT_Zero_Records_Query, @ParmDefinition, @sift_zero_rec output;
  if (Round ((cast(@sift_zero_rec AS Decimal(10,0)) / cast(@sift_total_rec AS DECIMAL(10,0))*100),0)) > 10 -- change this if you want a different percentage  
   BEGIN
/*
 On SQL Server 2005 and 2008 we whould get a better query plan with 'DELETE TOP(500) FROM TABLE WHERE ...'
 But I happen to service customers that are still using SQL Server 2000 so we use the old syntax
 You can of course change the number of rows to delete, this way it seams resonably fast 
 and it's possible to break out of the loop when @stoptime is reached
*/

   set @Delete_Statement = 'DELETE FROM dbo.[' + object_name(@sift_tab_id)+'] ' +@SIFT_Where_Clause
   declare @rowcount int
   select @rowcount = 1
   select @deleted_records = 0
   set ROWCOUNT 500 -- This decides how many rows to delete in each go
   	  while @rowcount > 0 and GETDATE() < @stoptime
	   begin
	    exec(@Delete_Statement)
	     set @rowcount = @@ROWCOUNT
         set @deleted_records = @deleted_records + @rowcount
       end
     print 'Deleted ' +cast(@deleted_records as varchar) +' records from the ' +@tname +' table'
	 select @total_deleted_records  =  @total_deleted_records + @deleted_records
/*
The we optimize the keys on the table we just deleted from, leaving 10% slack
*/	
	  DBCC DBREINDEX (@tname, '', 90) WITH NO_INFOMSGS 
	  IF @backup = '' goto End_Step
	  set @backupcmd = N'BACKUP LOG [' +DB_NAME() +'] ' +@backup
	  exec sp_executesql @backupcmd
	 
    END

End_Step:
  fetch next from sift_tab_cur into @sift_tab_id
END

--CLEANUP
print cast(@total_deleted_records as nvarchar) +' records deleted'
close sift_tab_cur
deallocate sift_tab_cur
drop table #sift_tab

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS ON 
GO

